/* FILE: oxsexcept.h                 -*-Mode: c++-*-
 *
 * Exception classes for Oxs.
 *
 */

#ifndef _OXS_EXCEPT
#define _OXS_EXCEPT

#include "oc.h"
#include <iostream>
#include <string>

OC_USE_STD_NAMESPACE;  // Specify std namespace, if supported

/* End includes */

// Dummy declarations that should move over to oommf/ext/oc
#define OXS_THROW(except,text)                                     \
   throw except(string("ERROR\n"                                   \
		       "   in file: " __FILE__ "\n"                \
		       "   at line: " OC_STRINGIFY(__LINE__) "\n") \
		+ text );

class Oc_Exception {
private:
  string msg;
public:
  Oc_Exception(string text);
  string MessageText() { return msg; }
  virtual const char* MessageType() =0;
  virtual ~Oc_Exception() {}
  void Prepend(const string& prefix) {
    msg = prefix + msg;
  }
};

// Oxs exceptions
class Oxs_Exception : public Oc_Exception {
public:
  Oxs_Exception(string text) : Oc_Exception(text) {}
};

// Bad pointer
class Oxs_BadPointer : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad pointer"; }
  Oxs_BadPointer(string text) : Oxs_Exception(text) {}
};

// Data structure lock action failure
class Oxs_BadLock : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad lock"; }
  Oxs_BadLock(string text) : Oxs_Exception(text) {}
};

// Data structure index invalid
class Oxs_BadIndex : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad data index"; }
  Oxs_BadIndex(string text) : Oxs_Exception(text) {}
};

// Resource allocation failure
class Oxs_BadResourceAlloc : public Oxs_Exception {
public:
  const char* MessageType() { return "Resource allocation failure"; }
  Oxs_BadResourceAlloc(string text) : Oxs_Exception(text) {}
};

// Resource deallocation failure
class Oxs_BadResourceDealloc : public Oxs_Exception {
public:
  const char* MessageType() { return "Resource deallocation failure"; }
  Oxs_BadResourceDealloc(string text) : Oxs_Exception(text) {}
};

// Overflow detected
class Oxs_Overflow : public Oxs_Exception {
public:
  const char* MessageType() { return "Overflow"; }
  Oxs_Overflow(string text) : Oxs_Exception(text) {}
};

// Bad parameter
class Oxs_BadParameter : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad parameter"; }
  Oxs_BadParameter(string text) : Oxs_Exception(text) {}
};

// Program logic error
class Oxs_ProgramLogicError : public Oxs_Exception {
public:
  const char* MessageType() { return "Program logic error"; }
  Oxs_ProgramLogicError(string text) : Oxs_Exception(text) {}
};

// No memory.  This exists for those broken C++ compilers (you
// know who you are) that don't throw a bad_alloc from new
class Oxs_NoMem : public Oxs_Exception {
public:
  const char* MessageType() { return "Insufficient memory"; }
  Oxs_NoMem(string text) : Oxs_Exception(text) {}
};

// Error reported by Tcl interpreter
class Oxs_TclError : public Oxs_Exception {
public:
  const char* MessageType() { return "Tcl error"; }
  Oxs_TclError(string text) : Oxs_Exception(text) {}

};

#endif // _OXS_EXCEPT

