/* FILE: atlasscalarfield.h      -*-Mode: c++-*-
 *
 * Atlas scalar field object, derived from Oxs_ScalarField
 * class.
 *
 */

#ifndef _OXS_ATLASSCALARFIELD
#define _OXS_ATLASSCALARFIELD

#include <string>
#include <vector>

#include "oc.h"

#include "atlas.h"
#include "scalarfield.h"

/* End includes */

// Struct for use inside Oxs_AtlasScalarField class only.
// Defined outside that class to please MSVC++ 5.0.
struct OxsASFSpecifiedValue {
  BOOL isset;
  REAL8m value;
  OxsASFSpecifiedValue() : isset(0), value(0.) {}
};
// The next 3 operators are defined so MSVC++ 5.0 will accept
// vector<OxsASFSpecifiedValue>, but are left undefined because
// they aren't needed.
BOOL operator<(const OxsASFSpecifiedValue&,const OxsASFSpecifiedValue&);
BOOL operator>(const OxsASFSpecifiedValue&,const OxsASFSpecifiedValue&);
BOOL operator==(const OxsASFSpecifiedValue&,const OxsASFSpecifiedValue&);

class Oxs_AtlasScalarField:public Oxs_ScalarField {
private:
  Oxs_OwnedPointer<Oxs_Atlas> atlas_obj; // Atlas basket
  Oxs_Key<Oxs_Atlas> atlas_key;
  vector<OxsASFSpecifiedValue> values;
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  Oxs_AtlasScalarField
  (const char* name,     // Child instance id
   Oxs_Director* newdtr, // App director
   const char* argstr);  // MIF input block parameters

  virtual ~Oxs_AtlasScalarField() {}

  virtual REAL8m Value(const ThreeVector& pt) const;

  virtual void FillMeshValue(const Oxs_Mesh* mesh,
			     Oxs_MeshValue<REAL8m>& array) const {
    DefaultFillMeshValue(mesh,array);
  }

};

#endif // _OXS_ATLASSCALARFIELD
