/* FILE: transformzeeman.h         -*-Mode: c++-*-
 *
 * Zeeman (applied field) energy, derived from Oxs_Energy class,
 * specified from a Tcl proc.  This is a generalization of the
 * Oxs_ScriptUZeeman and Oxs_FixedZeeman classes.
 *
 */

#ifndef _OXS_TRANSFORMZEEMAN
#define _OXS_TRANSFORMZEEMAN

#include <string>
#include <vector>

#include "director.h"
#include "energy.h"
#include "simstate.h"
#include "tclcommand.h"
#include "threevector.h"
#include "util.h"
#include "vectorfield.h"

/* End includes */

class Oxs_TransformZeeman:public Oxs_Energy {
private:
  vector<Oxs_TclCommandLineOption> command_options;
  Oxs_TclCommand cmd;

  REAL8m hmult;

  void GetAppliedField
  (const Oxs_SimState& state,
   ThreeVector& row1, ThreeVector& row2, ThreeVector& row3,
   ThreeVector& drow1, ThreeVector& drow2, ThreeVector& drow3) const;

  enum TransformType { identity, diagonal, symmetric, general };
  TransformType transform_type;

  mutable UINT4m mesh_id;
  Oxs_OwnedPointer<Oxs_VectorField> fixedfield_init;
  mutable Oxs_MeshValue<ThreeVector> fixedfield;
  /// fixedfield is a cached value filled by
  /// fixedfield_init when a change in mesh is
  /// detected.

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_TransformZeeman(const char* name,     // Child instance id
		      Oxs_Director* newdtr, // App director
		      const char* argstr);  // MIF input block parameters
  virtual ~Oxs_TransformZeeman();
  virtual BOOL Init();
};


#endif // _OXS_TRANSFORMZEEMAN
