/* FILE: uzeeman.h            -*-Mode: c++-*-
 *
 * Uniform Zeeman (applied field) energy, derived from Oxs_Energy class.
 *
 */

#ifndef _OXS_UZEEMAN
#define _OXS_UZEEMAN

#include <vector>
#include "threevector.h"
#include "output.h"
#include "energy.h"

/* End includes */

class Oxs_UZeeman:public Oxs_Energy {
private:
  vector<ThreeVector> Happ;  // Indexed off of state stage_number
  ThreeVector GetAppliedField(UINT4m stage_number) const;
  /// Utility function. Indexes Happ off of stage_number with
  /// proper range behavior.

  ThreeVector GetAppliedField(const Oxs_SimState& state) const;

  UINT4m FieldCount() const {
    return static_cast<UINT4m>(Happ.size());
  }

  // Supplied outputs, in addition to those provided by Oxs_Energy.
  Oxs_ScalarOutput<Oxs_UZeeman> Bapp_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappx_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappy_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappz_output;
  void Fill__Bapp_output(const Oxs_SimState& state);

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_UZeeman(const char* name,     // Child instance id
	      Oxs_Director* newdtr, // App director
	      const char* argstr);  // MIF input block parameters
  virtual ~Oxs_UZeeman();
  virtual BOOL Init();
  virtual void StageRequestCount(unsigned int& min,
				 unsigned int& max) const;
  // min is set to the size of Happ, i.e., the number of applied
  // field values.  If this value larger than 1, then max is set
  // to the same value, for tight stage range checking.
  // Otherwise, max is set to UINT_MAX, i.e., infinity.
  
};


#endif // _OXS_UZEEMAN
